# the purpose of this file is so we can run from the build tree.
# for installation, this is replaced with a better version.
import os

__all__ = ["__VERSION__", "GETTEXT_PACKAGE",
    "localedir", "pkgdatadir", "pythondir", "pkgconfdir", "libdir",
    "LIBSMBIOS_C_SONAME"
    ]

LIBSMBIOS_C_SONAME="libsmbios_c.so.2"

import libsmbios_c
_modpath = os.path.realpath(libsmbios_c.__path__[0])

# the following vars are all substituted on install
__VERSION__="2.4.3"
GETTEXT_PACKAGE="libsmbios-2.4-x86_64"
libdir="/usr/lib"
pythondir="/usr/lib/python3.14/site-packages"
localedir="/usr/share/locale"
pkgdatadir="/usr/share/smbios-utils"
pkgconfdir="/etc/libsmbios"

# end vars

